@@ -55,15 +55,14 @@
 	gr->gr_name = strdup (grent->gr_name);
 	/*@=mustfreeonly@*/
 	if (NULL == gr->gr_name) {
-		free(gr);
+		gr_free(gr);
 		return NULL;
 	}
 	/*@-mustfreeonly@*/
 	gr->gr_passwd = strdup (grent->gr_passwd);
 	/*@=mustfreeonly@*/
 	if (NULL == gr->gr_passwd) {
-		free(gr->gr_name);
-		free(gr);
+		gr_free(gr);
 		return NULL;
 	}
 
@@ -73,21 +72,13 @@
 	gr->gr_mem = (char **) malloc ((i + 1) * sizeof (char *));
 	/*@=mustfreeonly@*/
 	if (NULL == gr->gr_mem) {
-		free(gr->gr_passwd);
-		free(gr->gr_name);
-		free(gr);
+		gr_free(gr);
 		return NULL;
 	}
 	for (i = 0; grent->gr_mem[i]; i++) {
 		gr->gr_mem[i] = strdup (grent->gr_mem[i]);
 		if (NULL == gr->gr_mem[i]) {
-			int j;
-			for (j=0; j<i; j++)
-				free(gr->gr_mem[j]);
-			free(gr->gr_mem);
-			free(gr->gr_passwd);
-			free(gr->gr_name);
-			free(gr);
+			gr_free(gr);
 			return NULL;
 		}
 	}
@@ -96,6 +87,18 @@
 	return gr;
 }
 
+void gr_free_members (struct group *grent)
+{
+	if (NULL != grent->gr_mem) {
+		size_t i;
+		for (i = 0; NULL != grent->gr_mem[i]; i++) {
+			free (grent->gr_mem[i]);
+		}
+		free (grent->gr_mem);
+		grent->gr_mem = NULL;
+	}
+}
+
 void gr_free (/*@out@*/ /*@only@*/struct group *grent)
 {
 	free (grent->gr_name);
@@ -103,13 +106,36 @@ void gr_free (/*@out@*/ /*@only@*/struct group *grent)
 		memzero (grent->gr_passwd, strlen (grent->gr_passwd));
 		free (grent->gr_passwd);
 	}
-	if (NULL != grent->gr_mem) {
-		size_t i;
-		for (i = 0; NULL != grent->gr_mem[i]; i++) {
-			free (grent->gr_mem[i]);
-		}
-		free (grent->gr_mem);
-	}
+	gr_free_members(grent);
 	free (grent);
 }
 
+bool gr_append_member(struct group *grp, char *member)
+{
+	int i;
+
+	if (NULL == grp->gr_mem || grp->gr_mem[0] == NULL) {
+		grp->gr_mem = (char **)malloc(2 * sizeof(char *));
+		if (!grp->gr_mem) {
+			return false;
+		}
+		grp->gr_mem[0] = strdup(member);
+		if (!grp->gr_mem[0]) {
+			return false;
+		}
+		grp->gr_mem[1] = NULL;
+		return true;
+	}
+
+	for (i = 0; grp->gr_mem[i]; i++) ;
+	grp->gr_mem = realloc(grp->gr_mem, (i + 2) * sizeof(char *));
+	if (NULL == grp->gr_mem) {
+		return false;
+	}
+	grp->gr_mem[i] = strdup(member);
+	if (NULL == grp->gr_mem[i]) {
+		return false;
+	}
+	grp->gr_mem[i + 1] = NULL;
+	return true;
+}
