@@ -33,8 +33,6 @@
 
 #include <config.h>
 
-#ident "$Id$"
-
 #include <fcntl.h>
 #include <grp.h>
 #include <pwd.h>
@@ -45,6 +43,7 @@
 #include "defines.h"
 #include "groupio.h"
 #include "nscd.h"
+#include "sssd.h"
 #include "prototypes.h"
 
 #ifdef SHADOWGRP
@@ -67,6 +66,7 @@
  * Global variables
  */
 const char *Prog;
+FILE *shadow_logfd = NULL;
 
 static const char *grp_file = GROUP_FILE;
 static bool use_system_grp_file = true;
@@ -81,6 +81,7 @@ static bool gr_locked = false;
 /* Options */
 static bool read_only = false;
 static bool sort_mode = false;
+static bool silence_warnings = false;
 
 /* local function prototypes */
 static void fail_exit (int status);
@@ -157,6 +158,7 @@ static /*@noreturn@*/void usage (int status)
 	                "                                but do not change files\n"), usageout);
 	(void) fputs (_("  -R, --root CHROOT_DIR         directory to chroot into\n"), usageout);
 	(void) fputs (_("  -s, --sort                    sort entries by UID\n"), usageout);
+	(void) fputs (_("  -S, --silence-warnings        silence controversial/paranoid warnings\n"), usageout);
 	(void) fputs ("\n", usageout);
 	exit (status);
 }
@@ -192,18 +194,19 @@ static void process_flags (int argc, char **argv)
 {
 	int c;
 	static struct option long_options[] = {
-		{"help",      no_argument,       NULL, 'h'},
-		{"quiet",     no_argument,       NULL, 'q'},
-		{"read-only", no_argument,       NULL, 'r'},
-		{"root",      required_argument, NULL, 'R'},
-		{"sort",      no_argument,       NULL, 's'},
+		{"help",             no_argument,       NULL, 'h'},
+		{"quiet",            no_argument,       NULL, 'q'},
+		{"read-only",        no_argument,       NULL, 'r'},
+		{"root",             required_argument, NULL, 'R'},
+		{"silence-warnings", no_argument,       NULL, 'S'},
+		{"sort",             no_argument,       NULL, 's'},
 		{NULL, 0, NULL, '\0'}
 	};
 
 	/*
 	 * Parse the command line arguments
 	 */
-	while ((c = getopt_long (argc, argv, "hqrR:s",
+	while ((c = getopt_long (argc, argv, "hqrR:sS",
 	                         long_options, NULL)) != -1) {
 		switch (c) {
 		case 'h':
@@ -220,6 +223,9 @@ static void process_flags (int argc, char **argv)
 		case 's':
 			sort_mode = true;
 			break;
+		case 'S':
+			silence_warnings = true;
+			break;
 		default:
 			usage (E_USAGE);
 		}
@@ -455,7 +461,7 @@ static void compare_members_lists (const char *groupname,
 				break;
 			}
 		}
-		if (*other_pmem == NULL) {
+		if (!silence_warnings && *other_pmem == NULL) {
 			printf
 			    ("'%s' is a member of the '%s' group in %s but not in %s\n",
 			     *pmem, groupname, file, other_file);
@@ -835,6 +841,7 @@ int main (int argc, char **argv)
 	 * Get my name so that I can use it to report errors.
 	 */
 	Prog = Basename (argv[0]);
+	shadow_logfd = stderr;
 
 	(void) setlocale (LC_ALL, "");
 	(void) bindtextdomain (PACKAGE, LOCALEDIR);
@@ -869,7 +876,10 @@ int main (int argc, char **argv)
 	/* Commit the change in the database if needed */
 	close_files (changed);
 
-	nscd_flush_cache ("group");
+	if (!read_only) {
+		nscd_flush_cache ("group");
+		sssd_flush_cache (SSSD_DB_GROUP);
+	}
 
 	/*
 	 * Tell the user what we did and exit.
