@@ -31,19 +31,17 @@
  */
 
 #include <config.h>
-
-#ident "$Id$"
-
 #include <assert.h>
+#include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
 #include <grp.h>
 #include <pwd.h>
 #include <stdio.h>
-#include <stdio.h>
-#include <sys/stat.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <unistd.h>
 #ifdef ACCT_TOOLS_SETUID
 #ifdef USE_PAM
 #include "pam_defs.h"
@@ -61,10 +59,14 @@
 #ifdef	SHADOWGRP
 #include "sgroupio.h"
 #endif				/* SHADOWGRP */
+#ifdef WITH_SELINUX
+#include <selinux/selinux.h>
+#endif				/* WITH_SELINUX */
 #ifdef WITH_TCB
 #include <tcb.h>
 #include "tcbfuncs.h"
 #endif				/* WITH_TCB */
+#include "run_part.h"
 /*@-exitarg@*/
 #include "exitcodes.h"
 #ifdef ENABLE_SUBIDS
@@ -89,6 +91,7 @@
  * Global variables
  */
 const char *Prog;
+FILE *shadow_logfd = NULL;
 
 static char *user_name;
 static uid_t user_id;
@@ -150,8 +153,9 @@ static void usage (int status)
 	                  "\n"
 	                  "Options:\n"),
 	                Prog);
-	(void) fputs (_("  -f, --force                   force removal of files,\n"
-	                "                                even if not owned by user\n"),
+	(void) fputs (_("  -f, --force                   force some actions that would fail otherwise\n"
+	                "                                e.g. removal of user still logged in\n"
+	                "                                or files, even if not owned by the user\n"),
 	              usageout);
 	(void) fputs (_("  -h, --help                    display this help message and exit\n"), usageout);
 	(void) fputs (_("  -r, --remove                  remove home directory and mail spool\n"), usageout);
@@ -1012,6 +1016,7 @@ int main (int argc, char **argv)
 	 * Get my name so that I can use it to report errors.
 	 */
 	Prog = Basename (argv[0]);
+	shadow_logfd = stderr;
 	(void) setlocale (LC_ALL, "");
 	(void) bindtextdomain (PACKAGE, LOCALEDIR);
 	(void) textdomain (PACKAGE);
@@ -1143,6 +1148,10 @@ int main (int argc, char **argv)
 	{
 		const struct passwd *pwd;
 
+		if (run_parts ("/etc/shadow-maint/userdel-pre.d", user_name,
+				"userdel")) {
+			exit(1);
+		}
 		pw_open(O_RDONLY);
 		pwd = pw_locate (user_name); /* we care only about local users */
 		if (NULL == pwd) {
@@ -1342,6 +1351,10 @@ int main (int argc, char **argv)
 		user_cancel (user_name);
 	close_files ();
 
+	if (run_parts ("/etc/shadow-maint/userdel-post.d", user_name, "userdel")) {
+		exit(1);
+	}
+
 #ifdef WITH_TCB
 	errors += remove_tcbdir (user_name, user_id);
 #endif				/* WITH_TCB */
